package com.yidian.shenghuoquan.newscontent.ui.store

import android.content.Intent
import android.os.Bundle
import android.widget.Toast
import com.yidian.common.base.BaseActivity
import com.yidian.shenghuoquan.newscontent.databinding.ActivityStoreEditBinding
import com.yidian.shenghuoquan.newscontent.http.ApiService
import com.yidian.shenghuoquan.newscontent.utils.StorageUtil
import com.yidian.shenghuoquan.newscontent.widget.CommonTopBarView

/**
 * 新增/编辑门店
 */
class EditStoreActivity : BaseActivity<ActivityStoreEditBinding>(), CommonTopBarView.OnCommonTopBarActionCallback {

  // 是否为新建门店
  private var isAddStore: Boolean = false
  private var store = Store("", "", "", "", "")

  override fun createViewBinding(): ActivityStoreEditBinding {
    return ActivityStoreEditBinding.inflate(layoutInflater)
  }

  override fun getXPageName(): String {
    return ""
  }

  override fun init(savedInstanceState: Bundle?) {
    intent?.let {
      when (it.getSerializableExtra("from")) {
        fromAddStore -> {
          isAddStore = true
          viewBind.bvTopBar.setPageTitle("新建门店")
          initContent()
        }
        fromEditStore -> {
          isAddStore = false
          viewBind.bvTopBar.setPageTitle("编辑门店")
          initContent(it)
        }
      }
      initClick()
    }
  }

  private fun initContent(intent: Intent? = null) {
    if (intent == null) {
      viewBind.storePhoneNum.setText("")
      viewBind.storeAddressEdit.text = ""
      viewBind.storeName.setText("")
      store.shop_id = ""
      store.sub_shop_id = ""
    } else {
      viewBind.storePhoneNum.setText(intent.getStringExtra("phone"))
      viewBind.storeAddressEdit.text = intent.getStringExtra("address")
      viewBind.storeName.setText(intent.getStringExtra("shop_name"))
      store.shop_id = intent.getStringExtra("shop_id") ?: ""
      store.sub_shop_id = intent.getStringExtra("sub_shop_id") ?: ""
    }
  }

  private fun initClick() {
    viewBind.bvTopBar.setOnCommonTopBarCallback(null, this)
    viewBind.addressLayout.setOnClickListener {
      startActivityForResult(Intent(this@EditStoreActivity, SelectAddressActivity::class.java), 200)
    }
  }

  /**
   * 保存（新建和编辑）
   */
  override fun onDoAction() {
    if (isAddStore) {
      toAddStore()
    } else {
      toUpdateStore()
    }
  }

  private fun toAddStore() {
    val requestParams = HashMap<String, String?>()
    requestParams["life_account_id"] = StorageUtil.getLifeAccountId()
    requestParams["phone"] = viewBind.storePhoneNum.text.toString()
    requestParams["shop_name"] = viewBind.storeName.text.toString()
    requestParams["latitude"] = store.latitude
    requestParams["longitude"] = store.longitude
    requestParams["address"] = store.title
    ApiService.addStore(requestParams) {
      if (it) {
        Toast.makeText(this@EditStoreActivity, "保存成功", Toast.LENGTH_SHORT).show()
        finish()
      }
    }
  }

  private fun toUpdateStore() {
    val requestParams = HashMap<String, String?>()
    requestParams["sub_shop_id"] = store.sub_shop_id
    requestParams["life_account_id"] = StorageUtil.getLifeAccountId()
    requestParams["phone"] = viewBind.storePhoneNum.text.toString()
    requestParams["shop_id"] = store.shop_id
    requestParams["shop_name"] = viewBind.storeName.text.toString()
    requestParams["latitude"] = store.latitude
    requestParams["longitude"] = store.longitude
    requestParams["address"] = store.title
    ApiService.updateStore(requestParams) {
      if (it) {
        Toast.makeText(this@EditStoreActivity, "保存成功", Toast.LENGTH_SHORT).show()
        finish()
      }
    }
  }

  override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
    super.onActivityResult(requestCode, resultCode, data)
    if (resultCode == 200) {
      data?.let {
        viewBind.storeAddressEdit.text = it.getStringExtra("title")
        store.title = it.getStringExtra("title") ?: ""
        store.latitude = it.getStringExtra("latitude") ?: ""
        store.longitude = it.getStringExtra("longitude") ?: ""
      }
    }
  }

  companion object {
    const val fromAddStore = "fromAddStore"
    const val fromEditStore = "fromEditStore"
  }

  data class Store(
    var title: String,
    var latitude: String,
    var longitude: String,
    var shop_id: String,
    var sub_shop_id: String
  )
}
